# Generated by h2py from /usr/include/dlfcn.h
from TYPES import *
RTLD_LAZY = 0x00001
RTLD_NOW = 0x00002
RTLD_NOLOAD = 0x00004
RTLD_GLOBAL = 0x00100
RTLD_LOCAL = 0x00000
RTLD_PARENT = 0x00200
RTLD_GROUP = 0x00400
RTLD_WORLD = 0x00800
RTLD_NODELETE = 0x01000
RTLD_CONFGEN = 0x10000
RTLD_REL_RELATIVE = 0x00001
RTLD_REL_EXEC = 0x00002
RTLD_REL_DEPENDS = 0x00004
RTLD_REL_PRELOAD = 0x00008
RTLD_REL_SELF = 0x00010
RTLD_REL_WEAK = 0x00020
RTLD_REL_ALL = 0x00fff
RTLD_MEMORY = 0x01000
RTLD_STRIP = 0x02000
RTLD_NOHEAP = 0x04000
RTLD_CONFSET = 0x10000
RTLD_DI_LMID = 1
RTLD_DI_LINKMAP = 2
RTLD_DI_CONFIGADDR = 3
RTLD_DI_MAX = 3
